import os
import json


def read_secrets(filename: str = None) -> dict:
    if filename is None:
        base_dir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
        filename = os.path.join(base_dir, ".secrets/crawler_secrets.json")
    try:
        with open(filename, mode="r") as f:
            return json.loads(f.read())
    except FileNotFoundError:
        msg = f"Not found file in {filename}"
        raise FileNotFoundError(msg)


def get_env() -> tuple[bool, dict]:
    try:
        naver_api_key = os.getenv("NAVER_API_KEY")
        naver_secret_key = os.getenv("NAVER_SECRET_KEY")
        naver_customer_id = os.getenv("NAVER_CUSTOMER_ID")
        if all(
            value is None
            for value in [naver_api_key, naver_secret_key, naver_customer_id]
        ):
            return False, {}
        return True, {
            "NAVER_API_KEY": naver_api_key,
            "NAVER_SECRET_KEY": naver_secret_key,
            "NAVER_CUSTOMER_ID": naver_customer_id,
        }
    except Exception as e:
        print(e)
        return False, {}


def set_env(naver_api_key: str, naver_secret_key: str, naver_customer_id: int) -> dict:
    return {
        "NAVER_API_KEY": naver_api_key,
        "NAVER_SECRET_KEY": naver_secret_key,
        "NAVER_CUSTOMER_ID": naver_customer_id,
    }


def get_secrets(filename: str = None) -> dict:
    try:
        is_success_env, secrets = get_env()
        if is_success_env:
            return secrets

        secrets = read_secrets(filename)
        if secrets:
            return secrets
    except Exception:
        return {}
