import re
import html
from typing import Optional
import requests
from datetime import datetime, date


class GoogleTotalSearch:
    def __init__(self, keyword: str, start_date: str, end_date: str):
        self.keyword = keyword
        self.start_date = datetime.strptime(start_date, "%Y-%m-%d").date()
        self.end_date = datetime.strptime(end_date, "%Y-%m-%d").date()
        self.total_search_result = None
        self.is_available_date = None

    @property
    def is_available_date(self):
        return self._is_available_date

    @is_available_date.setter
    def is_available_date(self, value):
        if self.end_date < self.start_date:
            raise ValueError("End date must be greater than  or equal to start date")
        if self.start_date >= date.today():
            raise ValueError("Start date must be less than today")
        if self.end_date >= date.today():
            raise ValueError("End date must be less than today")
        self._is_available_date = True

    def get_search_url(self) -> str:
        start_date_string = datetime.strftime(self.start_date, "%m/%d/%Y")
        end_date_string = datetime.strftime(self.end_date, "%m/%d/%Y")
        url = (
            f"https://www.google.com/search?q={self.keyword}"
            f"&source=lnt&tbs=cdr:1,cd_min:{start_date_string},cd_max:{end_date_string}"
        )
        return url

    def extract(self) -> Optional[list[str]]:
        search_url = self.get_search_url()
        headers = {
            "Cache-Control": "max-age=0",
            "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) "
            "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.4951.54 Safari/537.36",
            "Accept-Language": "ko-KR,ko;q=0.9,en-US;q=0.8,en;q=0.7",
        }
        response = requests.get(search_url, headers=headers)
        if response.status_code == 200:
            split_pattern = r'data-async-context="query:'
            try:
                split_html = re.split(split_pattern, response.text)
                pattern = r'<div.*?data-hveid=.*?><a href="(.*?)".*?><br><h3 class=".*?">(.*?)</h3>'
                prog = re.compile(pattern)
                result = prog.findall(split_html[1])
                if result:
                    return result
                else:
                    return None
            except Exception:
                return None
        else:
            print("fail: google total search response is not 200")
            return None

    def get_processing_data(self) -> tuple[bool, str]:
        data = self.extract()
        result = []
        if data:
            try:
                for item in data:
                    result.append({"link": item[0], "title": html.unescape(item[1])})
                self.total_search_result = result
                return True, "find"
            except Exception as e:
                print(e)
                return False, "exception"
        else:
            return False, "fail: extract"
