import re
import html
from datetime import datetime, date
from typing import Optional

import requests


class NaverTotalSearch:
    def __init__(self, keyword: str, start_date: str, end_date: str):
        self.keyword = keyword
        self.start_date = datetime.strptime(start_date, "%Y-%m-%d").date()
        self.end_date = datetime.strptime(end_date, "%Y-%m-%d").date()
        self.total_search_result = None
        self.is_available_date = None

    @property
    def is_available_date(self):
        return self._is_available_date

    @is_available_date.setter
    def is_available_date(self, value):
        if self.end_date < self.start_date:
            raise ValueError("End date must be greater than or equal to start date")
        if self.start_date > date.today():
            raise ValueError("Start date must be less than or equal to today")
        if self.end_date > date.today():
            raise ValueError("End date must be less than or equal to today")
        self._is_available_date = True

    def get_search_url(self) -> str:
        start_date_string = datetime.strftime(self.start_date, "%Y%m%d")
        end_date_string = datetime.strftime(self.end_date, "%Y%m%d")
        url = (
            f"https://search.naver.com/search.naver?display=15&f=&filetype=0"
            f"&nso=so:r,a:all,p:from{start_date_string}to{end_date_string}"
            f"&page=2&query={self.keyword}&research_url=&sm=tab_pge&start=1&where=web"
        )
        return url

    def extract(self) -> Optional[list[str]]:
        search_url = self.get_search_url()
        response = requests.get(search_url)
        if response.status_code == 200:
            pattern = r'<div class="total_tit"> <a target="_blank" href="(.*?)" class="link_tit".*?> (.*?) </a> </div>'
            prog = re.compile(pattern)
            result = prog.findall(response.text)
            if result:
                return result
            else:
                return None
        else:
            print("fail: naver total search response is not 200")
            return None

    def get_processing_data(self) -> tuple[bool, str]:
        data = self.extract()
        result = []
        if data:
            try:
                title_pattern = r"|".join((r"<(.*?)> ", r"<mark>", r"</mark>"))
                for item in data:
                    title = re.sub(title_pattern, "", item[1])
                    result.append({"link": item[0], "title": html.unescape(title)})
                self.total_search_result = result
                return True, "find"
            except Exception as e:
                print(e)
                return False, "exception"
        else:
            return False, "fail: extract"
